
//=============================================================================
//  Microsoft (R) Bloodhound (tm). Copyright (C) 1991-1992.
//
//  MODULE: struct.h
//
//  Description:
//
//  This source file contains data structure definitions.
//
//  Modification History
//
//=============================================================================

#pragma pack(1)

//=============================================================================
//  Forward references.
//=============================================================================

typedef struct _CAPTURE      *LPCAPTURE;

typedef struct _BTE          *LPBTE;

//=============================================================================
//  Parser data structure.
//=============================================================================

typedef struct _PARSER
{
    BYTE                FileName[16];           //... Parser DLL file name.
    HANDLE              hModule;                //... Handle to parser DLL.
    DWORD               State;                  //... Current state of parser.
} PARSER;

typedef PARSER  *LPPARSER;

#define PARSER_SIZE     sizeof(PARSER)

//=============================================================================
//  Parser states
//=============================================================================

#define PARSER_STATE_INITIALIZED    1       //... parser has been created.
#define PARSER_STATE_LOADED         2       //... parser has been loaded.
#define PARSER_STATE_UNLOADED       3       //... parser has been unloaded.

//=============================================================================
//  Parser table (master parser table).
//=============================================================================

typedef struct _PARSERTABLE
{
    DWORD               nParsers;       //... number of parsers in list.
    PARSER              Parser[0];      //... parser table.
} PARSERTABLE;

typedef PARSERTABLE  *LPPARSERTABLE;

#define PARSERTABLE_SIZE    sizeof(PARSERTABLE)

//=============================================================================
//  Protocol data structure.
//=============================================================================

typedef struct _PROTOCOL
{                                               
    OBJECTTYPE          ObjectType;             //... Must be first member.
    LPPARSER            Parser;                 //... Parser which owns this protocol.
    DWORD               State;                  //... Protocol flags.
    PROTOCOLINFO        ProtocolInfo;           //... Public info about protocol.
    LPPROTOCOLTABLE     FollowSet;              //... The follow set for this protocol. NULL if empty.

    //...   Protocol entry points passed to CreateProtocol().

    REGISTER            Register;
    DEREGISTER          Deregister;
    RECOGNIZEFRAME      RecognizeFrame;
    ATTACHPROPERTIES    AttachProperties;
    FORMATPROPERTIES    FormatProperties;
} PROTOCOL;
                    
typedef PROTOCOL *LPPROTOCOL;

#define PROTOCOL_SIZE   sizeof(PROTOCOL)

//=============================================================================
//  Protocol states
//=============================================================================

#define PROTOCOL_STATE_INITIALIZED    1       //... protocol has been created.
#define PROTOCOL_STATE_REGISTERED     2       //... protocol has been registered.
#define PROTOCOL_STATE_DEREGISTERED   3       //... protocol has been deregistered.

//=============================================================================
//  Property data structure.
//
//  WARNING: This data structure is used to convert a DWORD property handle
//           into a easy-to-use structure rather than dinking around (ed) with LOWORD()
//           and HIWORD() macros.
//
//  DO NOT ADD NEW MEMBERS TO THIS STRUCTURE!!!!!
//=============================================================================

typedef struct _PROPERTY
{
    WORD    PropertyID;                             //... Index into property database.
    WORD    ProtocolID: 15;                         //... Index into protocol table.
    WORD    ValidProperty: 1;                       //... If 0 the property is not valid.
} PROPERTY;
                    
typedef PROPERTY *LPPROPERTY;

#define PROPERTY_SIZE   sizeof(PROPERTY)            //... THIS BETTER BE A DWORD (2 WORD's)!!!!!

//=============================================================================
//  Address database structure.
//=============================================================================

typedef struct _ADDRESSDB
{
    OBJECTTYPE          ObjectType;             //... Must be first member.
    HOBJECTHEAP         hAddressInfoHeap;       //... heap of AddressInfo structures
    LPVOID              InstanceData;           //... instance data 
    DWORD               nTableSize;             //... size of lpAddressInfoTable in entries
    LPADDRESSINFOTABLE  lpAddressTable;         //... AddressInfoTable (sorted by address)
    LPADDRESSINFOTABLE  lpNameTable;            //... AddressInfoTable (sorted by name)
} ADDRESSDB;

typedef ADDRESSDB *LPADDRESSDB;

#define ADDRESSDB_SIZE      sizeof(ADDRESSDB)

//=============================================================================
//  Capture file header.
//=============================================================================

#define CAPTUREFILE_SIGNATURE       MAKE_IDENTIFIER('R', 'T', 'S', 'S')

typedef struct _CAPTUREFILE_HEADER
{
    DWORD           Signature;          //... Unique identifier: 'RTSS'.
    BYTE            BCDVerMinor;        //... Binary coded decimal (minor).
    BYTE            BCDVerMajor;        //... Binary coded decimal (major).
    WORD            MacType;            //... Topology type.
    SYSTEMTIME      TimeStamp;          //... time of capture.
    DWORD           FrameTableOffset;   //... Frame index table.
    DWORD           FrameTableLength;   //... Frame index table size.
    DWORD           UserDataOffset;     //... User data offset.
    DWORD           UserDataLength;     //... User data length.
    DWORD           CommentDataOffset;  //... Comment Data offset
    DWORD           CommentDataLength;  //... Length of comment data.
    DWORD           StatisticsOffset;   //....offset to STATISTICS STRUCTURE
    DWORD           StatisticsLength;   //....length of stats struct
    DWORD           NetworkInfoOffset;  //....offset to network info structure
    DWORD           NetworkInfoLength;  //....length of network info structure
} CAPTUREFILE_HEADER;

typedef CAPTUREFILE_HEADER  *LPCAPTUREFILE_HEADER;

#define CAPTUREFILE_HEADER_SIZE sizeof(CAPTUREFILE_HEADER)

//=============================================================================
//  Capture file.
//=============================================================================

typedef struct _CAPTUREFILE
{
    union
    {
        CAPTUREFILE_HEADER  Header;
        BYTE                Buffer[CAPTUREFILE_HEADER_SIZE];
    };

    BYTE                    Reserved[128 - CAPTUREFILE_HEADER_SIZE];
} CAPTUREFILE;

typedef CAPTUREFILE         *LPCAPTUREFILE;

#define CAPTUREFILE_SIZE     sizeof(CAPTUREFILE)

//=============================================================================
//  Frame descriptor (one for each frame in a capture context).
//  HFRAME is a pointer to a FRAMEDESC!
//=============================================================================

#ifdef DEBUG
#define FRAME_SIGNATURE          MAKE_IDENTIFIER('F', 'R', 'M', '$')
#endif

typedef struct _FRAMEDESC
{
#ifdef DEBUG
    DWORD                Signature;         //... FRM$
#endif

    DWORD                FrameNumber;            //... Frame Number.
    ULPFRAME             Frame;                  //... pointer to frame.
    LPPROPERTYTABLE      lpPropertyTable;        //... pointer to property table.
    LPRECOGNIZEDATATABLE lpRecognizeDataTable;   //... pointer to table of RECOGNIZEDATA structures.
    LPCAPTURE            lpCapture;              //... capture context.
    BYTE                 FrameLock;              //... frame lock count.
    BYTE                 PropertyTableLock;      //... property text lock count.
    WORD                 Flags;                  //... flags.
} FRAMEDESC;

typedef FRAMEDESC  *LPFRAMEDESC;

#define FRAMEDESC_SIZE  sizeof(FRAMEDESC)

#define FRAMEDESC_FLAGS_FRAME_INITIALIZED       0x0001
#define FRAMEDESC_FLAGS_FRAME_RECOGNIZED        0x0002
#define FRAMEDESC_FLAGS_PROPERTIES_ATTACHED     0x0004
#define FRAMEDESC_FLAGS_FRAME_FORMATTED         0x0008
#define FRAMEDESC_FLAGS_CREATE_FRAME            0x0010

//=============================================================================
//  Frame table.
//=============================================================================

typedef struct _FRAMETABLE
{
    DWORD           nLockedFrames;              //... Number of locked frames.
    DWORD           nValidFrames;               //... Number of valid frame descriptors.
    DWORD           nFrames;                    //... Number of frames.
    LPFRAMEDESC     FrameDesc[0];               //... Array of frame descriptor pointers.
} FRAMETABLE;

typedef FRAMETABLE *LPFRAMETABLE;

#define FRAMETABLE_SIZE     sizeof(FRAMETABLE)

//=============================================================================
//  Capture context.
//=============================================================================

typedef struct _CAPTURE
{
    OBJECTTYPE      ObjectType;             //... Must be first member.

    //=========================================================================
    //  Frame table.
    //=========================================================================

    LPFRAMETABLE    FrameTable;             //... Frame index table.

    //=========================================================================
    //  General items.
    //=========================================================================

    DWORD           Flags;                  //... Capture flags
    LPVOID          InstData;               //... Instance data of creator.
    HADDRESSDB      hAddressDB;             //... AddressDB for this capture context.
    SYSTEMTIME      TimeStamp;              //... Time of capture.

    DWORD           nEnabledProtocols;      //... Number of enabled protocols.
    LPBOOL          ProtocolEnabled;        //... Used at recognized time to varify a protocol is enabled.

    LPPROTOCOL      MacProtocol;            //... Our MAC-level protocol.
    LPPROTOCOLTABLE EnabledProtocolTable;   //... Table of enabled protocols for GetEnabledProtocols() API.
    HANDLE          hCCHeap;                //... Capture Context heap handle
    LPVOID   *      ProtocolCCInstPtr;      //... Array of Pointers to Capture Context Instance
                                            //... ProtocolCCInstPtr[nEnabledProtocols]

    //=========================================================================
    //  Capture file items.
    //=========================================================================

    DWORD           MacType;                //... Network MAC type.
    DWORD           MaxFrameSize;           //... Network Max frame size.
    DWORD           FileType;               //... Capture file type.
    DWORD           FileSize;               //... Size of loaded capture file.
    HBUFFER         hBuffer;                //... Current capture buffer.
    HANDLE          hFile;                  //... Capture file handle
    HANDLE          hFileMapping;           //... Handle to file mapping.
    LPCAPTUREFILE   CaptureFile;            //... Pointer to capture file.
} CAPTURE;

#define CAPTURE_SIZE  sizeof(CAPTURE)

typedef struct _RawAddressType
{
    union
    {
        BYTE MACAddress[MAC_ADDRESS_SIZE];
        BYTE IPAddress[IP_ADDRESS_SIZE];
        BYTE IPXRawAddress[IPX_ADDRESS_SIZE];
    };
} RawAddressType;


#pragma pack()
